<?php
/* Created on : Apr 3, 2011
 * Author     : Josph Tinsley - twitter.com/josephtinsley
 * Description: Google Analytics Class.
 * Makes connection to API and pull accounts list and reporting data.
 */
Class GoogleAnalyticsAPI
{
    private $login_url     = 'https://www.google.com/accounts/ClientLogin';
    private $account_url   = 'https://www.google.com/analytics/feeds/accounts/default';
    private $reporting_url = 'https://www.google.com/analytics/feeds/data';
    private $token;


    public function  __construct($email, $password)
    {

        $url    = $this->login_url;
        $login_creditials = array(
                          'accountType' => 'GOOGLE',
                          'Email' =>  $email,
                          'Passwd' => $password,
                          'service' => 'ANALYTICS',
                          'source' => 'Joseph_Tinsley_GATest-1.00' // NAME OF YOUR APP
                          );

        $conn_response = $this->curlConnection($url, $login_creditials, $header_info = NULL);

        if($conn_response['header_status_code'] == 200)
        {
            $this->setAccessToken($conn_response['auth_request']);
        }
        else
        {
           
            echo $this->catchError('CLIENT CONNECTION FAILED');
        }

    }


    private function curlConnection($url, $login_creditials, $header_info)
    {

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

    switch ($login_creditials)
    {
        case !NULL:
                curl_setopt($ch, CURLOPT_POST, TRUE);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $login_creditials);

                $output_data    = curl_exec($ch);    // TOKEN DATA
                $response_info  = curl_getinfo($ch); // HEADER CODE
                curl_close($ch);

                return array('header_status_code' => $response_info['http_code'],
                             'auth_request' => $output_data);
                 break;

        case NULL:
                curl_setopt($ch, CURLOPT_HTTPHEADER, $header_info);

                $output_data    = curl_exec($ch);    // XML DATA
                $response_info  = curl_getinfo($ch); // HEADER CODE
                curl_close($ch);

                return array('header_status_code' => $response_info['http_code'],
                             'account_data' => $output_data);
                break;
    }


    }

    private function setAccessToken($auth_request)
    {

        $output_data_array = explode("\n", trim($auth_request) );
        $this->token = $output_data_array[2]; // THIS IS OUR TOKEN. AND WE NEED THIS TO DO ANYHITNG WITH THE API
    }


    private function setHeaderInfo()
    {
        return $header_info = array('Authorization: GoogleLogin '. $this->token);
    }


    public function getAnalyticsAccounts()
    {
        $header_info = $this->setHeaderInfo();
        $url         = $this->account_url;
        $conn_response = $this->curlConnection($url, $login_creditials = NULL, $header_info);

        if($conn_response['header_status_code'] == 200)
        {

            $xml_data = $this->parseXML($conn_response['account_data']);
            return $xml_data;
        }
        else
        {
           
            echo $this->catchError('FAILED ACCOUNT CONNECTION');
        }

    }


    private function parseXML($account_data)
    {
        $xml = simplexml_load_string($account_data);
        return $xml;
    }

    public function showAnalyticsAccounts($account_list)
    {

        $results .= '<Table id="account_table">' ."\n";
        $results .= '<tr id="header">' ."\n";
        $results .= '<td>'. $account_list->title . '</td>' ."\n";
        $results .= '</tr>' ."\n";
        $results .= '<tr id="header">' ."\n";
        $results .= '<td>Number of Accounts ' . count($account_list->entry) . '</td>' ."\n";
        $results .= '</tr>' ."\n";

        for($x = 0; $x < count($account_list->entry); $x++)
        {

        $results .= '<tr id="highlight">' ."\n";
        $results .= '<td onclick="link(\'?account_id='.str_replace('http://www.google.com/analytics/feeds/accounts/','',$account_list->entry[$x]->id ).'\');">'. $account_list->entry[$x]->title . ' </td>' . "\n";
        $results .= '</tr>' ."\n";

        }
        $results .= '</Table>' ."\n";
        return $results;
    }



    public function getReportingData($account_id)
    {
        /* THERE IS SO MUCH THAT CAN BE DONE HERE.
         * ADJUST THERE PARAMTERES TO GATHER DIFFERNT DATA VEIWS
         * http://code.google.com/apis/analytics/docs/gdata/gdataReferenceDimensionsMetrics.html
         */
        $filter_params = array(
                        'ids'=>"$account_id",
                        'start-date' => date("Y-m-d", strtotime("-32 days") ),
                        'end-date' => date("Y-m-d", strtotime("-2 days") ),
                        'dimensions'=>'ga:source,ga:medium,ga:keyword,ga:browser,ga:region,ga:city',
                        'metrics'=>'ga:visits,ga:bounces',
                        'sort'=>'-ga:visits',
                        'max-results'=>25,
                        'prettyprint'=>TRUE
                        );

        $header_info = $this->setHeaderInfo();
        $url         = $this->reporting_url;

        $post_query_string = http_build_query($filter_params);
        $url = $url . '?' . $post_query_string;
        $conn_response = $this->curlConnection($url, $login_creditials = NULL, $header_info);

        if($conn_response['header_status_code'] == 200)
        {

            $xml_data = $this->parseXML($conn_response['account_data']);
            return $xml_data;
        }
        else
        {

            echo $this->catchError('FAILED REPORT RETRIEVING');
            
        }
        

    }

    public function showReportingdata($reporting_data)
    {
        //SOURCE | MEDIUM | KEYWORD |  BROWSER | CITY | REGION
        $results .= '<Table id="report_table">' ."\n";
        $results .= '<tr id="header">' ."\n";
        $results .= '<td colspan="6">'. $reporting_data->title . '</td>';

        $results .= '<tr id="header">' ."\n";
        $results .= '<td>Source</td>';
        $results .= '<td>Medium</td>';
        $results .= '<td>Keyword</td>';
        $results .= '<td>Browser</td>';
        $results .= '<td>City</td>';
        $results .= '<td>Region</td>' ."\n";
        $results .= '</tr>' ."\n";

        for($x = 0; $x < count($reporting_data->entry); $x++)
        {

        list($source, $medium, $keyword, $browser, $city, $region) = explode('|', $reporting_data->entry[$x]->title );

        $results .= '<tr id="highlight">' ."\n";
        $results .= '<td>'. str_replace('ga:source=','',trim($source)) . '</td>';
        $results .= '<td>'. str_replace('ga:medium=','',trim($medium)) . '</td>';
        $results .= '<td>'. str_replace('ga:keyword=','',trim($keyword)) . '</td>';
        $results .= '<td>'. str_replace('ga:browser=','',trim($browser)) . '</td>';
        $results .= '<td>'. str_replace('ga:region=','',trim($city)) . '</td>';
        $results .= '<td>'. str_replace('ga:city=','',trim($region)) . '</td>';
        $results .= '</tr>' ."\n";

        }
        $results .= '</Table>' ."\n";
        return $results;
    }

    private function catchError($error)
    {
          return 'Message: ' .$error . "<br />";
    }

}
?>